package net.knarcraft.blockhunt;

import net.knarcraft.blockhunt.arena.Arena;
import net.knarcraft.blockhunt.arena.ArenaState;
import net.knarcraft.blockhunt.manager.MessageManager;
import org.bukkit.Location;
import org.bukkit.block.Sign;
import org.bukkit.event.block.SignChangeEvent;

import java.util.List;

public class SignsHandler {

    public static void createSign(SignChangeEvent event, String[] lines, Location location) {
        if (lines[1] != null) {
            if (lines[1].equalsIgnoreCase("leave")) {
                saveSign("leave", location);
            } else if (lines[1].equalsIgnoreCase("shop")) {
                saveSign("shop", location);
            } else {
                boolean saved = false;
                for (Arena arena : MemoryStorage.arenaList) {
                    if (lines[1].equals(arena.arenaName)) {
                        saveSign(arena.arenaName, location);
                        saved = true;
                    }
                }

                if (!saved) {
                    MessageManager.sendFMessage(event.getPlayer(), ConfigC.error_noArena, "name-" + lines[1]);
                }
            }
        }
    }

    private static void saveSign(String key, Location location) {
        boolean saved = false;
        int number = 1;
        while (!saved) {
            if (MemoryStorage.signs.getFile().get(key + "_" + number) == null) {
                MemoryStorage.signs.getFile().set(key + "_" + number + ".arenaName", key);
                MemoryStorage.signs.getFile().set(key + "_" + number + ".location", location);
                MemoryStorage.signs.save();

                saved = true;
            } else {
                number = number + 1;
            }
        }
    }

    public static void removeSign(Location location) {
        //TODO: This seems really inefficient
        for (String sign : MemoryStorage.signs.getFile().getKeys(false)) {
            Location loc = (Location) MemoryStorage.signs.getFile().get(sign + ".location");
            if (loc.equals(location)) {
                MemoryStorage.signs.getFile().set(sign, null);
                MemoryStorage.signs.save();
            }
        }
    }

    public static boolean isSign(Location location) {
        for (String sign : MemoryStorage.signs.getFile().getKeys(false)) {
            Location loc = (Location) MemoryStorage.signs.getFile().get(sign + ".location");
            if (loc == null) {
                throw new IllegalArgumentException("Sign location could not be loaded.");
            }
            if (loc.equals(location)) {
                return true;
            }
        }

        return false;
    }

    public static void updateSigns() {
        //TODO: Instead of this awful method of signs updating every second for no reason, it would be preferable for 
        // sign changes to be directly triggered when its contents changes 
        MemoryStorage.signs.load(); //TODO: This loads from disk once every second which seems unnecessary
        for (String sign : MemoryStorage.signs.getFile().getKeys(false)) {
            Location location = (Location) MemoryStorage.signs.getFile().get(sign + ".location");
            if (location == null) {
                throw new IllegalArgumentException("Sign location could not be loaded.");
            }
            if (location.getWorld() == null) {
                throw new IllegalArgumentException("Sign world is null!");
            }

            // check if that area is actually loaded. If not move on.
            if (!location.getWorld().isChunkLoaded(location.getBlockX() >> 4, location.getBlockZ() >> 4)) {
                continue;
            }
            if (location.getBlock().getState() instanceof Sign signBlock) {
                String[] lines = signBlock.getLines();
                if (sign.contains("leave")) {
                    drawSign(MemoryStorage.config.getFile().getStringList(ConfigC.sign_LEAVE.location), signBlock);
                } else if (sign.contains("shop")) {
                    drawSign(MemoryStorage.config.getFile().getStringList(ConfigC.sign_SHOP.location), signBlock);
                } else {
                    drawArenaSign(lines[1], signBlock);
                }
            } else {
                removeSign(location);
            }
        }
    }

    private static void drawArenaSign(String arenaName, Sign signBlock) {
        for (Arena arena : MemoryStorage.arenaList) {
            //Find the correct arena
            if (!arenaName.endsWith(arena.arenaName)) {
                continue;
            }
            if (arena.gameState.equals(ArenaState.WAITING)) {
                drawSign(MemoryStorage.config.getFile().getStringList(ConfigC.sign_WAITING.location), signBlock, arena);
            } else if (arena.gameState.equals(ArenaState.STARTING)) {
                drawSign(MemoryStorage.config.getFile().getStringList(ConfigC.sign_STARTING.location), signBlock, arena);
            } else if (arena.gameState.equals(ArenaState.IN_GAME)) {
                drawSign(MemoryStorage.config.getFile().getStringList(ConfigC.sign_INGAME.location), signBlock, arena);
            }
        }
    }

    private static void drawSign(List<String> signLines, Sign signBlock, Arena arena) {
        drawSign(signLines, signBlock, "arenaname-" + arena.arenaName, "players-" + arena.playersInArena.size(),
                "maxplayers-" + arena.maxPlayers, "timeleft-" + arena.timer);
    }

    private static void drawSign(List<String> signLines, Sign signBlock, String... variables) {
        int lineCount = 0;
        for (String line : signLines) {
            if (lineCount <= 3) {
                signBlock.setLine(lineCount, MessageManager.replaceAll(line, variables));
            }

            lineCount = lineCount + 1;
        }
        signBlock.update();
    }

}
